/*
** Copyright (C) 1999, 2000 ABiSoft, Inc.
** All Rights Reserved.
** 
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of ABiSoft, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of ABiSoft, Inc.
** 
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
*/

#include "IrcAddOn.h"

IrcAddOn::IrcAddOn(BLooper *server_connection)
{
	fServerConnection = server_connection;
	fMessenger = BMessenger(fServerConnection);
	fNick = NULL;
}

IrcAddOn::~IrcAddOn()
{
}

bool IrcAddOn::HandleUserEntry(const char *entry, const char *chan, IrcView *view, bool *deleted_view, BMessage *msg)
{
	return false;
}

uint32 IrcAddOn::HandleEvent(uint32 type, char *raw, uint32 previously, BMessage *msg)
{
	return NOT_DISPLAYED;
}

bool IrcAddOn::LooperNeedsQuit()
{
	return false;
}

void IrcAddOn::SendToServer(const char *buf)
{
	BMessage msg(SEND_TO_SERVER);
	msg.AddString("buf", buf);
	fServerConnection->PostMessage(&msg); // the looper is the IrcMainWindow..
								// it receives the message and passes it on
								// to the fServerConnection
}

void IrcAddOn::PlaySound(int32 which, char *checkIdentifier)
{
	BMessage msg(PLAY_SOUND);
	msg.AddInt32("which", which);
	if(checkIdentifier)
		msg.AddString("CheckIdentifier", checkIdentifier);
	fServerConnection->PostMessage(&msg);
}

void IrcAddOn::SetNick(char *nick)
{
	fNick = nick;	
}

void IrcAddOn::SetHostAddress(char *host_address)
{
	fHostAddress = host_address;	
}

void IrcAddOn::SetValues(bool joins, bool parts, bool quits,
		bool notices, bool nickchanges, bool irc_color, bool mirc_color,
		bool auto_chat, bool auto_send)
{
	fJoins = joins;
	fParts = parts;
	fQuits = quits;
	fNotices = notices;
	fNickChanges = nickchanges;
	fIrcColor = irc_color;
	fMircColor = mirc_color;
	fAutoChat = auto_chat;
	fAutoSend = auto_send;
}

void IrcAddOn::SetHighlightList(BListView *list)
{
	fHighlightList = list;
}

void IrcAddOn::SetTime(char *time_string, bool time_stamp, bool message_stamp)
{
	fTime = time_string;
	fTimeStamp = time_stamp;
	fMessageStamp = message_stamp;
}

void IrcAddOn::SetFontsAndColors(BFont serv, BFont nick, 
	BFont text, BList *list)
{
	fServerFont = serv;
	fNickFont = nick;
	fTextFont = text;

	rgb_color *color = (rgb_color *)list->ItemAt(3);
	fTextColor = *color;
	
	color = (rgb_color *)list->ItemAt(4);
	fServerColor = *color;
	
	color = (rgb_color *)list->ItemAt(5);
	fActionColor = *color;
	
	color = (rgb_color *)list->ItemAt(6);
	fWhoisColor = *color;
	
	color = (rgb_color *)list->ItemAt(7);
	fHighlightColor = *color;
	
	color = (rgb_color *)list->ItemAt(8);
	fNickColor = *color;
	
	color = (rgb_color *)list->ItemAt(9);
	fCtcpColor = *color;
	
	color = (rgb_color *)list->ItemAt(11);
	fOpColor = *color;

	color = (rgb_color *)list->ItemAt(12);
	fPlusVColor = *color;
	
	color = (rgb_color *)list->ItemAt(13);
	fUserColor = *color;

	color = (rgb_color *)list->ItemAt(17);
	fOwnNickColor = *color;

	color = (rgb_color *)list->ItemAt(18);
	fUrlColor = *color;
	
	color = (rgb_color *)list->ItemAt(19);
	fNickChangeColor = *color;
	
	color = (rgb_color *)list->ItemAt(20);
	fJoinColor = *color;
	
	color = (rgb_color *)list->ItemAt(21);
	fPartColor = *color;
	
	color = (rgb_color *)list->ItemAt(22);
	fQuitColor = *color;
}

BMenuItem *IrcAddOn::MenuItem()
{
	return NULL;
}

void IrcAddOn::HandleMenuItem(BMenuItem *mi, BList *users)
{
	// this function never gets called if IrcAddOn::MenuItem() returns NULL
}