/*
** Copyright (C) 1999, 2000 ABiSoft, Inc.
** All Rights Reserved.
** 
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of ABiSoft, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of ABiSoft, Inc.
** 
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
*/

#ifndef IRC_ADD_ON_H
#define IRC_ADD_ON_H

#include <Messenger.h>
#include <Looper.h>
#include "IrcEvents.h"
#include <GraphicsDefs.h>
#include <Font.h>
#include "IrcView.h"

// Messages are posted to each AddOn whenever an event occurs
// Messages have type IRC_UPDATE
// Add-ons are only responsible for what happens on one server
// A new copy of each add-on is instantiated for each server connection

class IrcAddOn
{
public:
	virtual ~IrcAddOn();
	
	// returns the name of the add-on
	virtual char *Name() = 0;
	// returns the add-ons description. Author, version, and help 
	virtual char *Description() = 0;
	// When the user types /join or /ignore or /dcc send
	// Baxter calls HandleUserEntry, regardless if Baxter knows how
	// to handle the command or not.
	// HandleUserEntry should return true if it can handle it.
	// In cases in which Baxter does not know how to handle the command,
	// but the add-on does, if the add-on returns true Baxter will _not_
	// send the command raw to the server.
	// Channel is the channel is the channel the command was initiated from
	// If it was a query, channel is the nick of the query.
	// ALERT! Don't call SendMessage anymore. Instead, put the message into the msg that gets passed to you through
	// HandleUserEntry
	virtual bool HandleUserEntry(const char *entry, const char *channel, IrcView *view, bool *deleted_view, BMessage *msg);
	// The previously uint32 is kind of tricky. If the HandleEvent function of an add-on
	// called before your add-on (right before or awhile before) returns DISPLAYED, 
	// your HandleEvent function will be passed previously = DISPLAYED. If you're 
	// HandleEvent function, for example, shows whois info and type == WHOIS_XXXXXX
	// then you'll want to check to see if the whois was already displayed (previously will be DISPLAYED)
	// and display it if previously == NOT_DISPLAYED
	// the last add-on called will always be the default Baxter add-on. It does nothing if
	// previously == DISPLAYED, as should other add-ons that can display anything and everything. 
	// THIS IS VERY, VERY IMPORTANT!!!!!!!!!
	// you are now allowed to change raw, but only mangle it in productive ways if you don't intend to display it.
	// For example, a pig-latin add-on could pig-latinize each PRIVMSG command, but pass it along to the default
	// Baxter add-on for displaying.
	virtual uint32 HandleEvent(uint32 type, char *raw, uint32 previously, BMessage *msg);
	// Some add ons are loopers, and they need to be quit in a certain way. Baxter takes care of this
	// for you if you return true, but only return true if you're a BLooper! (Otherwise don't implement this func)
	virtual bool LooperNeedsQuit();
	void SetNick(char *nick);
	void SetTime(char *time, bool time_stamp, bool message_stamp);
	void SetFontsAndColors(BFont text_font, BFont nick_font, BFont server_font, BList *list);
	void SetHostAddress(char *host_address);
	void SetValues(bool joins, bool parts, bool quits,
		bool notices, bool nickchanges, bool irc_color, bool mirc_color,
		bool auto_chat, bool auto_send);
	void SetHighlightList(BListView *list);
	// the which choices are defined in IrcEvents.h.. each is preceded by SOUND_

	// This is an optional function. It can return a BMenuItem (or a BMenu).
	// When the menu item is invoked Baxter will call add-on->
	// HandleMenuItem();
	virtual BMenuItem *MenuItem();
	// The BList has at least one char * item in it representing the nicks
	// that were selected when the menuitem was invoked
	// The BList belongs to the add-on when it gets called, 
	// but the strings in the BList do not. Don't delete the strings, delete
	// the BList.
	virtual void HandleMenuItem(BMenuItem *mi, BList *users);
protected:
	// Don't create an IrcAddOn directly!
	IrcAddOn(BLooper *server_connection);
	void SendToServer(const char *buf);
	
	// the checkIdentifier is usually a channel or the nick of a user you have a query
	// open with. If the sound is set to play when the sound event occurs in a channel
	// that has focus, checkIdentifier is used to check whether that channel has focus
	void PlaySound(int32 which, char *checkIdentifier = NULL);
	char *fNick, *fTime, *fHostAddress;
	rgb_color fNickColor, fServerColor, fActionColor, fTextColor,
		fOpColor, fPlusVColor, fUserColor, fWhoisColor, fHighlightColor,
		fOwnColor, fCtcpColor, fOwnNickColor, fUrlColor,
		fNickChangeColor, fJoinColor, fPartColor, fQuitColor;
	BFont fTextFont, fNickFont, fServerFont;
	bool fTimeStamp;
	bool fIrcColor, fMircColor, fAutoChat, fAutoSend;
	// These are set in prefs. If fJoins is false for example, don't
	// report joins to the user!
	bool fJoins, fParts, fQuits, fNotices, fNickChanges, fMessageStamp;
	// Only look at items in this list, never mess with them.
	BListView *fHighlightList;
private:
	BLooper *fServerConnection;
	BMessenger fMessenger;
};

#if __MWERKS__
#pragma export on
#endif

const float SUPPORTED_VERSION_NUMBER = 1.3;

extern "C" {
_EXPORT IrcAddOn *instantiate(BLooper *server_connection);
_EXPORT	extern float ADD_ON_VERSION;
}

#if __MWERKS__
#pragma export reset
#endif

#endif // IRC_ADD_ON_H
