/*
** Copyright (C) 1999, 2000 ABiSoft, Inc.
** All Rights Reserved.
** 
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of ABiSoft, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of ABiSoft, Inc.
** 
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
*/

// This is the general purpose view for display... ChannelView, ServerView,
// and PrivateView all derive from it.

#ifndef IRC_VIEW_H
#define IRC_VIEW_H

#include <View.h>
#include <TextView.h>
//#include "IrcServer.h"
//#include "IrcTextView.h" // since all the IrcView derivatives use this,
						 // why not include it here

enum irc_type {
	server,
	channel,
	query,
	channel_list,
	notify_list,
	dcc_view
};

class ServerConnection;

class IrcView : public BView
{
public:
	IrcView(BRect frame, const char *name, uint32 resizeMask, uint32 flags, irc_type type);
	virtual void EventHandle(int32 type, char *buf, char *buf2 = NULL, 
		char *buf3 = NULL, char *buf4 = NULL, char *buf5 = NULL) = 0;
	virtual void Append(char *append, text_run_array *runs, bool contains_nick = false) = 0;
	virtual void SetSeparateWindow(bool separate) = 0;
	virtual bool InSeparateWindow() = 0;
	virtual ServerConnection *GetServerConnection() = 0;
	irc_type Type() { return fType; };
	virtual void SetTransition(bool transition) = 0;
	virtual int32 CompletedAppends() = 0;
	virtual bool RequestingLock() = 0;
	virtual void Clear() = 0;
	virtual void SetColors() = 0;
	virtual BList *SelectedUsers() = 0;
	virtual void SetSkins(BBitmap *textViewBack, BBitmap *mainViewBack, BBitmap *listViewBack);
	virtual void SetViewSkins() = 0;
protected:
	BBitmap *fTextViewBack, *fMainViewBack, *fListViewBack;
private:
	irc_type fType;
};

const char NOTIFICATION_CHARACTER = 15; // at the beginning of user joined notifications, and user online notifications
const uint32 NOTIFICATION_CLICKED_ON = 'clik';

const char *const urlPrefixes[] = {
	"http://",
	"ftp://",
	"shttp://",
	"https://",
	"finger://",
	"telnet://",
	"gopher://",
	"news://",
	"nntp://",
	"www.",
	"file:/",
	"mailto:",
	"@",
	NULL
};

#endif