/*
** Copyright (C) 1999, 2000 ABiSoft, Inc.
** All Rights Reserved.
** 
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of ABiSoft, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of ABiSoft, Inc.
** 
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
*/

#include "TTT.h"
#include "AddOnVersion.h"
#include "TTTWindow.h"
#include <Application.h>
#include <errno.h>

IrcAddOn *instantiate(BLooper *server_connection)
{
	return new TTT(server_connection);
}

TTT::TTT(BLooper *server_connection) :
	IrcAddOn(server_connection)
{
	fQuit = false;
}


TTT::~TTT()
{
	fQuit = true;
	
	int32 count = fWindowList.CountItems();
	BWindow *window;
	
	for(int32 i = count - 1; i >= 0; i--) {
		window = (BWindow *)fWindowList.ItemAt(i);
		window->Lock();
		window->Quit();
	}
}

bool TTT::LooperNeedsQuit()
{
	return false;
}

bool TTT::HandleUserEntry(const char *entry, const char *channelName, IrcView *view, bool *deleted_view, BMessage *message)
{
	if(strncasecmp(entry, "/TTT", 4) == 0) {
		int32 length = strlen(entry);
		if(length > 5) {
			string parse(entry, 5, length - 5);

			ttt_send *send = new ttt_send;
	
			send->nick = parse;
			send->add_on = this;
				
			thread_id id = spawn_thread(_send, "TTT Initiate", B_NORMAL_PRIORITY, send);
			resume_thread(id);
		}
		return true;
	}
	return false;
}

#include <StopWatch.h>

uint32 TTT::HandleEvent(uint32 type, char *raw, uint32 previously, BMessage *message)
{
	if(previously == DISPLAYED)
		return NOT_DISPLAYED;
	if(type == CTCP_UNKNOWN && strstr(raw, "TTT") != NULL) {
		int32 plc = 1;
		
		char sender[256];
		strcpy(sender, &raw[1]);
		
		stop_at(sender, '!');
		plc += strlen(sender);
		
		while(raw[plc] != ':' && raw[plc] != '\0')
			plc++;
		// get to the first ':'
		// now skip the words "TTT" (3 chars long + 3 for spaces)
		plc += 6;
		
		int32 length = strlen(raw);
		string request(raw, plc, length);
		
		char buf[1024];
		sprintf(buf, "%s wants to play Tic-Tac-Toe with you.", sender);
				
		//BInvoker *invoker = new BInvoker(msg, this, this);
		BAlert *alert = new BAlert("", buf, "Refuse", "Accept", NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT);
		if(alert->Go() == 1) {			
			int32 spc = request.find(" ");
			
			string ip_string(request, 0, spc);
			
			spc++;
			
			string port_string(request, spc, request.length() - spc - 1);
			
			uint32 ip = strtoul(ip_string.c_str(), (char **)NULL, 10);
			uint32 port_num = strtoul(port_string.c_str(), (char **)NULL, 10);
			
			ttt_recv *recv = new ttt_recv;
			thread_id *id = new thread_id;
						
			recv->ip = ip;
			recv->port = port_num;
			recv->add_on = this;
			recv->nick = sender;
						
			*id = spawn_thread(_recv, "TTT Receive", B_NORMAL_PRIORITY, recv);
			resume_thread(*id);
		}
			
		return DISPLAYED;
	}
	return NOT_DISPLAYED;
}

char *TTT::Name()
{
	return "Tic-Tac-Toe";
}

char *TTT::Description()
{
	return "By Seth Flaxman (seth@abisoft.com)";
}

void TTT::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case ACCEPT_TTT: {
			const char *request_string = NULL;
		
			if(msg->FindString("request", (const char **)&request_string) != B_OK)
				return;
			string request(request_string);
			
			entry_ref ref;
			msg->FindRef("directory", &ref);
			BEntry entry(&ref);
			BPath path;
			entry.GetPath(&path);
			
			const char *name;
			msg->FindString("name", (const char **)&name);
			
			path.Append(name);
			
			int32 spc = request.find(" ");
			
			int32 next_spc = request.find(" ", spc+1);
			string ip_string(request, spc+1, next_spc);
			
			spc = next_spc+1;
			next_spc = request.find(" ", spc+1);
			string port_string(request, spc, next_spc);
			
			spc = next_spc+1;
			next_spc = request.find(" ", spc+1);
			string size_string(request, spc, next_spc);
			
			uint32 ip = strtoul(ip_string.c_str(), (char **)NULL, 10);
			uint32 port_num = strtoul(port_string.c_str(), (char **)NULL, 10);
			
			ttt_recv *recv = new ttt_recv;
			thread_id *id = new thread_id;
						
			recv->ip = ip;
			recv->port = port_num;
			recv->add_on = this;
						
			*id = spawn_thread(_recv, "TTT Receive", B_NORMAL_PRIORITY, recv);
			resume_thread(*id);
		}
			break;
		case SEND_FILE: {
			entry_ref ref;
			const char *nick = NULL;
			msg->FindString("nick", (const char **)&nick);
			
			msg->FindRef("refs", &ref);
			BEntry entry(&ref);
			BPath path;
			entry.GetPath(&path);

			ttt_send *send = new ttt_send;
			thread_id *id = new thread_id;
			
			send->nick = nick;
			send->add_on = this;
						
			*id = spawn_thread(_send, "TTT Send", B_NORMAL_PRIORITY, send);
			resume_thread(*id);
		}
			break;
	}
}


status_t TTT::_recv(void *arg)
{
	ttt_recv *recv = (ttt_recv *)arg;
	recv->add_on->Receive(*recv);
	delete recv;
	return B_OK;
}

void TTT::Receive(ttt_recv info)
{
	int32 sock = socket(AF_INET, SOCK_STREAM, 0);
	if(sock < 0) {
		// error creating socket!
		return;
	}
	struct sockaddr_in server;
	
	memset(&server,0,sizeof(server));
	server.sin_family=AF_INET;
	server.sin_addr.s_addr=htonl(info.ip);
	server.sin_port=htons((short)info.port);
	if(connect(sock, (struct sockaddr*)&server, sizeof(server)) < 0) {
		// no connect!
		printf("No Connect!\n");
		return;
	}

	// start the TTTWindow with sock here
	// pass it the fWindowList, so it can remove and add itself as necessary
	
	TTTWindow *window = new TTTWindow(&sock, info.nick.c_str(), O, &fWindowList);
	
	char c;
	char tonum[2];
	tonum[1] = '\0';
	
	while(fQuit == false) {
		if(sock != -1 && recv(sock,&c,1, 0) == 1) {
			if(c == 'q') { // time to disconnect
				window->Disconnected();
				closesocket(sock);
				break;
			}
			
			tonum[0] = c;
			int32 move = atoi(tonum);
			
			window->MarkSquare(move);
		}
		else if(errno == EINTR) {
			// we got interupted by a send call... keep trying to receive
		}
		else {
			// we've been disconnected
			// if it was us that did the disconnecting, then TTTWindow is no longer
			// in the list (it removed itself), and calling window->Disconnected() will crash
			if(fWindowList.HasItem(window) && fQuit == false) {
				window->Disconnected();
			}
			break;
		}
	}
}

status_t TTT::_send(void *arg)
{
	ttt_send *send = (ttt_send *)arg;
	send->add_on->Send(*send);
	delete send;
	return B_OK;
}

void TTT::Send(ttt_send info)
{
	// random port number
	int port=23800+rand()%100;
	printf("random port: %d\n", port);
	hostent *hp = gethostbyname(fHostAddress);

	// retrieve host name
	if(!hp) {
		return;
	}
	
	// setup socket for listening
	int sock = socket(AF_INET,SOCK_STREAM,0);
	if(sock < 0) {
		return;
	}

	struct sockaddr_in server;
	memset(&server,0,sizeof(server));
	server.sin_family=AF_INET;
	server.sin_addr.s_addr=(INADDR_ANY);
	server.sin_port=(port);

	if(bind(sock,(struct sockaddr*)&server,sizeof(server))==-1) {
		return;
	}
	
	char buf[512];
		
	sprintf(buf, "PRIVMSG %s :%cTTT %u %d\001\r\n", info.nick.c_str(), 
		1, htonl(((struct in_addr*)(hp->h_addr_list)[0])->s_addr),
		htons(port));

	SendToServer(buf);	
	
	listen(sock,4);
	struct sockaddr_in remadr;
	int siz=sizeof(remadr);

	int32 sock2 = accept(sock,(struct sockaddr*)&remadr,&siz);

	// start as X	
	
	TTTWindow *window = new TTTWindow(&sock2, info.nick.c_str(), X, &fWindowList);
	
	char c;
	char tonum[2];
	tonum[1] = '\0';
	
	while(fQuit == false) {
		if(sock2 != -1 && recv(sock2,&c,1, 0) == 1) {
			if(c == 'q') { // time to disconnect
				window->Disconnected();
				closesocket(sock2);
				break;
			}
			tonum[0] = c;
			int32 move = atoi(tonum);
			
			window->MarkSquare(move);
		}
		else if(errno == EINTR) {
			// we got interupted by a send call... keep trying to receive
		}
		else {
			// we've been disconnected
			// if it was us that did the disconnecting, then TTTWindow is no longer
			// in the list (it removed itself), and calling window->Disconnected() will crash
			if(fWindowList.HasItem(window) && fQuit == false) {
				window->Disconnected();
			}
			break;
		}
	}
}

BMenuItem *TTT::MenuItem()
{
	return new BMenuItem("Play Tic-Tac-Toe", new BMessage);
}

void TTT::HandleMenuItem(BMenuItem *mi, BList *users)
{
	char buf[1024];
	int32 count = users->CountItems();
	bool b;
	BMessage msg;
	
	for(int32 i = 0; i < count; i++) {
		sprintf(buf, "/TTT %s", ((char *)users->ItemAt(i)));
		HandleUserEntry(buf, "", NULL, &b, &msg);
	}
	
	delete users;
}

void TTT::stop_at(char *buf, char stopc)
{
	int i = 0;
	while(buf[i] != '\0' && buf[i] != stopc)
		i++;
	buf[i] = '\0';
}

