#include "TTTView.h"
#include <Alert.h>
#include <Message.h>

TTTView::TTTView(BRect frame) : BView(frame, "TTTView", B_FOLLOW_ALL, B_WILL_DRAW)
{	
	for(int32 x = 0; x < 3; x++) {
		for(int32 y = 0; y < 3; y++) {
			fMarks[x][y] = blank;
		}
	}
	
	fClickRestarts = false;
	
	frame.OffsetTo(B_ORIGIN); // just in case
	
	fDraw = new BBitmap(frame, B_RGB32, true);
	fDrawView = new BView(frame, "fDrawView", B_FOLLOW_NONE, B_WILL_DRAW);

	fSquare = frame;
	fSquare.right /= 3;
	fSquare.bottom /= 3;
	
	fDraw->AddChild(fDrawView);
	
	DrawLines();

	BRect rect = fSquare;
	rect.InsetBy(1,1);
	
	fX = new BBitmap(rect, B_RGB32, true);
	fO = new BBitmap(rect, B_RGB32, true);
	
	BView *temp = new BView(rect, "temp", B_FOLLOW_NONE, B_WILL_DRAW);
	
	fX->Lock();
	fX->AddChild(temp);
	temp->SetFontSize(50);
	temp->SetHighColor(255,0,0);
	temp->DrawString("X", BPoint(5, fSquare.bottom - 5));
	temp->Sync();
	fX->RemoveChild(temp);
	fX->Unlock();
	
	fO->Lock();
	fO->AddChild(temp);
	temp->SetFontSize(50);
	temp->SetHighColor(0,0,255);
	temp->DrawString("O", BPoint(5, fSquare.bottom - 5));
	temp->Sync();
	fO->RemoveChild(temp);
	fO->Unlock();
	
	delete temp;
}

TTTView::~TTTView()
{
	delete fX;
	delete fO;
	delete fDraw;
}

void TTTView::MouseDown(BPoint pt)
{
	if(Window()->IsActive() == false) {
		uint32 buttons;
		GetMouse(&pt, &buttons);
		if(buttons == B_PRIMARY_MOUSE_BUTTON) {
			Window()->Activate();
			return;
		}
	}
	
	if(fClickRestarts) {
		for(int32 x = 0; x < 3; x++) {
			for(int32 y = 0; y < 3; y++) {
				fMarks[x][y] = blank;
			}
		}
		fDraw->Lock();
		fDrawView->FillRect(fDrawView->Bounds(), B_SOLID_LOW);
		fDrawView->Sync();
		fDraw->Unlock();
		
		DrawLines();
		Draw(BRect());
	
		fClickRestarts = false;
	}
	else {
		int32 x = pt.x / (Frame().right / 3);
		int32 y = pt.y / (Frame().bottom / 3);
			
		if(fMarks[x][y] == blank) {
			// now change the x and y to a number between 0 and 8
			int32 clicked = (x) + y * 3;
			
			
			BMessage msg(SQUARE_CLICKED);
			msg.AddInt32("clicked", clicked);
			
			Window()->PostMessage(&msg);
		}
	}
}

void TTTView::Draw(BRect)
{
	DrawBitmap(fDraw, B_ORIGIN);
}

void TTTView::MarkSquare(int32 which, int32 m)
{
	int32 x = which % 3;
	int32 y = which / 3;
	
	if(fClickRestarts)
		MouseDown(BPoint(0,0));
	if(fMarks[x][y] != blank)
		return;

	BPoint pt(x * fSquare.right + 1, y * fSquare.bottom + 1);
	
	fDraw->Lock();
	
	fMarks[x][y] = m;
	
	if(m == X) {
		fDrawView->DrawBitmap(fX, pt);	
	}
	else {
		fDrawView->DrawBitmap(fO, pt);	
	}
	
	fDrawView->Sync();
	fDraw->Unlock();
	
	Draw(BRect());

	CheckForWin(x, y);
}

void TTTView::CheckForWin(int32 x, int32 y)
{
	bool match = false;
	int32 m = fMarks[x][y];
	
	for(int32 y = 0; y < 3; y++) { // horizontal
		if((fMarks[0][y] + fMarks[1][y] + fMarks[2][y]) == m * 3) {
			match = true;
			break;
		}
	}
	
	if(!match) {
		for(int32 x = 0; x < 3; x++) {
			if((fMarks[x][0] + fMarks[x][1] + fMarks[x][2]) == m * 3) {
				match = true;
				break;
			}
		}
	
		if(!match) {
			if((fMarks[0][0] + fMarks[1][1] + fMarks[2][2]) == m * 3)
				match = true;
			else if((fMarks[0][2] + fMarks[1][1] + fMarks[2][0]) == m * 3)
				match = true;
		}
	}
	
	bool cats_game = false;
	
	if(!match) { // if no match, maybe it's a cats game
		cats_game = true;
		
		for(int32 x = 0; x < 3; x++) {
			for(int32 y = 0; y < 3; y++) {
				if(fMarks[x][y] == blank)
					cats_game = false;
			}
		}
	}
	
	if(match || cats_game) {
		int32 choice;
		
		fDraw->Lock();
		
		BRect bounds = fDrawView->Bounds();
		font_height fh;
		be_plain_font->GetHeight(&fh);
		
		float height = fh.ascent + fh.descent + 5;
		
		BPoint leftTop(5, (bounds.bottom - height) * .5);
		
		BRect rect(5, leftTop.y, bounds.right - 5, leftTop.y + height);
		fDrawView->SetHighColor(255,0,0);
		fDrawView->FillRect(rect);
		
		fDrawView->SetHighColor(255,255,255);
		fDrawView->SetLowColor(255,0,0);
	
		if(m == X && cats_game == false) {
			fDrawView->MovePenTo((rect.right - be_plain_font->StringWidth("X Wins (click to play again)")) * .5, rect.bottom - 5);
			fDrawView->DrawString("X Wins (click to play again)");
		}
		else if(m == O && cats_game == false) {
			fDrawView->MovePenTo((rect.right - be_plain_font->StringWidth("O Wins (click to play again)")) * .5, rect.bottom - 5);
			fDrawView->DrawString("O Wins (click to play again)");
		}
		else if(cats_game) {
			fDrawView->MovePenTo((rect.right - be_plain_font->StringWidth("Cat's Game (click to play again)")) * .5, rect.bottom - 5);
			fDrawView->DrawString("Cat's Game (click to play again)");
		}
		
		fDrawView->SetHighColor(0,0,0);		
		fDrawView->SetLowColor(255,255,255);
		fDrawView->Sync();
		fDraw->Unlock();
		
		Draw(BRect());
		
		fClickRestarts = true;
	}
}

void TTTView::DrawLines()
{
	fDraw->Lock();
	
	BRect frame = fDrawView->Bounds();
	
	fDrawView->SetPenSize(1.0);
	fDrawView->StrokeRect(frame);
	
	fDrawView->StrokeLine(BPoint(0, fSquare.bottom), BPoint(frame.right, fSquare.bottom));
	fDrawView->StrokeLine(BPoint(0, (fSquare.bottom * 2)), BPoint(frame.right, (fSquare.bottom * 2)));
	
	fDrawView->StrokeLine(BPoint(fSquare.right, 0), BPoint(fSquare.right, frame.bottom));
	fDrawView->StrokeLine(BPoint((fSquare.right * 2), 0), BPoint((fSquare.right * 2), frame.bottom));
	
	fDrawView->Sync();
	fDraw->Unlock();
}

void TTTView::Disconnected()
{
	fDraw->Lock();
	
	BRect frame = fDrawView->Bounds();
		
	font_height fh;
	be_plain_font->GetHeight(&fh);
	
	float height = fh.ascent + fh.descent + 5;
	
	BPoint leftTop(5, (frame.bottom - height) * .5);
	
	BRect rect(5, leftTop.y, frame.right - 5, leftTop.y + height);
	fDrawView->SetHighColor(255,0,0);
	fDrawView->FillRect(rect);
	
	fDrawView->SetHighColor(255,255,255);
	fDrawView->SetLowColor(255,0,0);

	fDrawView->MovePenTo((rect.right - be_plain_font->StringWidth("Connection Closed")) * .5, rect.bottom - 5);
	fDrawView->DrawString("Connection Closed");
	
	fDrawView->SetHighColor(0,0,0);		
	fDrawView->SetLowColor(255,255,255);

	fDrawView->Sync();
	fDraw->Unlock();
	
	Draw(BRect());
	printf("Disconnected!\n");
}