#include "TTTWindow.h"
#include "TTTView.h"
#include <Message.h>

TTTWindow::TTTWindow(int32 *socket, const char *opponent, int32 usermark, BList *list) : 
	BWindow(BRect(100,100,300,300), "", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_WILL_ACCEPT_FIRST_CLICK)
{
	fWindowList = list;
	fSocket = socket;
	fUserMark = usermark;
	fOpponent = opponent;
	
	fView = new TTTView(Bounds());
	AddChild(fView);
	
	if(usermark == X)
		fTurn = true;
	else
		fTurn = false;
	
	UpdateTitle();
	
	fWindowList->AddItem(this);
	
	Show();
}

bool TTTWindow::QuitRequested()
{
	return true;
}

void TTTWindow::Quit()
{
	fWindowList->RemoveItem(this);
	
	if(fSocket) {
		char buf[2];
		sprintf(buf, "q");
		send(*fSocket, buf, 1, 0);
		int sock = *fSocket;
		*fSocket = -1;
		
		closesocket(sock);
	}
	
	BWindow::Quit();
}

void TTTWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case SQUARE_CLICKED: {
			if(fTurn) { // if it's the users turn
				int32 clicked;
				msg->FindInt32("clicked", &clicked);
				
				char buf[16];
				sprintf(buf, "%ld", clicked);
				send(*fSocket, buf, strlen(buf), 0);
				
				fView->MarkSquare(clicked, fUserMark);
				fTurn = fTurn?false:true;
				
				UpdateTitle();
			}
		}
			break;
		default:
			BWindow::MessageReceived(msg);
	}
}

void TTTWindow::MarkSquare(int32 which)
{
	Lock();
	if(fUserMark == O)
		fView->MarkSquare(which, X);
	else 
		fView->MarkSquare(which, O);
		
	// if fTurn was false, make it true, and vice verse	
	// the "a ? b : c" construct
	// works just like if(a) b; else c;
	fTurn = fTurn?false:true;

	UpdateTitle();

	Unlock();
}

void TTTWindow::UpdateTitle()
{
	BString title(fOpponent);
	if(fTurn == true) {
		title += " (your turn)";
	}
	else {
		title += " (waiting)";
	}
	
	SetTitle(title.String());
}

void TTTWindow::Disconnected()
{
	Lock();
	fSocket = NULL;
	fView->Disconnected();
	Unlock();
}